//
// Created by lukas on 06.03.21.
//

#pragma once

#include <HttpEndpoint.h>
#include <WebSocketTxRx.h>
#include "Clock.h"

#define DEFAULT_CLOCK_STATE true
#define OFF_STATE "OFF"
#define ON_STATE "ON"

#define CLOCK_ON 0x1
#define CLOCK_OFF 0x0


#define LIGHT_SETTINGS_ENDPOINT_PATH "/rest/lightState"
#define LIGHT_SETTINGS_SOCKET_PATH "/ws/lightState"

class WordClockState {
 public:
  bool wordClockOn;

  static void read(WordClockState& settings, JsonObject& root) {
    root["led_on"] = settings.wordClockOn;
  }

  static StateUpdateResult update(JsonObject& root, WordClockState& lightState) {
    boolean newState = root["led_on"] | DEFAULT_CLOCK_STATE;
    if (lightState.wordClockOn != newState) {
      lightState.wordClockOn = newState;
      return StateUpdateResult::CHANGED;
    }
    return StateUpdateResult::UNCHANGED;
  }

  static void haRead(WordClockState& settings, JsonObject& root) {
    root["state"] = settings.wordClockOn ? ON_STATE : OFF_STATE;
  }

  static StateUpdateResult haUpdate(JsonObject& root, WordClockState& lightState) {
    String state = root["state"];
    // parse new led state
    boolean newState = false;
    if (state.equals(ON_STATE)) {
      newState = true;
    } else if (!state.equals(OFF_STATE)) {
      return StateUpdateResult::ERROR;
    }
    // change the new state, if required
    if (lightState.wordClockOn != newState) {
      lightState.wordClockOn = newState;
      return StateUpdateResult::CHANGED;
    }
    return StateUpdateResult::UNCHANGED;
  }
};

class ClockService : public StatefulService<WordClockState> {
 public:
  ClockService(AsyncWebServer* server, SecurityManager* securityManager);
  void begin();

 private:
  HttpEndpoint<WordClockState> _httpEndpoint;
  WebSocketTxRx<WordClockState> _webSocket;

  Clock clock;

  void onConfigUpdated();
};

